@@ -1,7 +1,7 @@
 /* fmfconv_ppm.c: ppm output routine included into fmfconv.c
-   Copyright (c) 2004-2005 Gergely Szasz
+   Copyright (c) 2004-2015 Gergely Szasz
 
-   $Id: fmfconv_ppm.c 4777 2012-11-26 23:35:48Z sbaldovi $
+   $Id: fmfconv_ppm.c 5424 2016-04-29 13:27:54Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,15 +30,18 @@
 #include "libspectrum.h"
 #include "fmfconv.h"
 
+extern int greyscale;
 
 int
 out_write_ppm( void )
 {
-  fprintf( out, "P6\n#ZX Spectrum screenshot created by fmfconv "
-		"(http://fuse-emulator.sourceforge.net) "
-		"\n%u %u 255\n", frm_w, frm_h );
+  fprintf( out, ( greyscale ? "P5\n" : "P6\n" ) );
+  fprintf( out, "#ZX Spectrum screenshot created by fmfconv "
+           "(http://fuse-emulator.sourceforge.net)\n"
+           "%u %u 255\n", frm_w, frm_h );
 
-  if( fwrite( pix_rgb, frm_w * frm_h * 3, 1, out ) != 1 ) return ERR_WRITE_OUT;
+  if( fwrite( pix_rgb, frm_w * frm_h * ( greyscale ? 1 : 3 ), 1, out ) != 1 )
+    return ERR_WRITE_OUT;
 
   printi( 2, "out_write_ppm()\n" );
 
