@@ -1,7 +1,7 @@
 /* rzxdump.c: get info on an RZX file
-   Copyright (c) 2002-2003 Philip Kendall
+   Copyright (c) 2002-2015 Philip Kendall, Sergio Baldovi
 
-   $Id: rzxdump.c 4767 2012-11-18 18:49:54Z sbaldovi $
+   $Id: rzxdump.c 5424 2016-04-29 13:27:54Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <getopt.h>
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
@@ -42,10 +43,14 @@
 
 #include "utils.h"
 
+#define PROGRAM_NAME "rzxdump"
+
 char *progname;
 
 const char *rzx_signature = "RZX!";
 
+size_t block_number;
+
 libspectrum_word read_word( unsigned char **ptr );
 libspectrum_dword read_dword( unsigned char **ptr );
 
@@ -80,20 +85,91 @@ read_dword( unsigned char **ptr )
   return result;
 }
 
+static void
+show_version( void )
+{
+  printf(
+    PROGRAM_NAME " (" PACKAGE ") " PACKAGE_VERSION "\n"
+    "Copyright (c) 2002-2014 Philip Kendall\n"
+    "License GPLv2+: GNU GPL version 2 or later "
+    "<http://gnu.org/licenses/gpl.html>\n"
+    "This is free software: you are free to change and redistribute it.\n"
+    "There is NO WARRANTY, to the extent permitted by law.\n" );
+}
+
+static void
+show_help( void )
+{
+  printf(
+    "Usage: %s [OPTION] <rzxfile>...\n"
+    "List contents of Sinclair ZX Spectrum input recording files.\n"
+    "\n"
+    "Options:\n"
+    "  -h, --help     Display this help and exit.\n"
+    "  -V, --version  Output version information and exit.\n"
+    "\n"
+    "Report %s bugs to <%s>\n"
+    "%s home page: <%s>\n"
+    "For complete documentation, see the manual page of %s.\n",
+    progname,
+    PROGRAM_NAME, PACKAGE_BUGREPORT, PACKAGE_NAME, PACKAGE_URL, PROGRAM_NAME
+  );
+}
 
 int main( int argc, char **argv )
 {
-  int i;
+  int i, c;
+  int error = 0;
+
+  struct option long_options[] = {
+    { "help", 0, NULL, 'h' },
+    { "version", 0, NULL, 'V' },
+    { 0, 0, 0, 0 }
+  };
 
   progname = argv[0];
 
-  if( argc < 2 ) {
+  while( ( c = getopt_long( argc, argv, "hV", long_options, NULL ) ) != -1 ) {
+
+    switch( c ) {
+
+    case 'h':
+      show_help();
+      return 0;
+
+    case 'V':
+      show_version();
+      return 0;
+
+    case '?':
+      /* getopt prints an error message to stderr */
+      error = 1;
+      break;
+
+    default:
+      error = 1;
+      fprintf( stderr, "%s: unknown option `%c'\n", progname, (char) c );
+      break;
+
+    }
+  }
+  argc -= optind;
+  argv += optind;
+
+  if( error ) {
+    fprintf( stderr, "Try `%s --help' for more information.\n", progname );
+    return error;
+  }
+
+  if( argc < 1 ) {
     fprintf( stderr, "%s: Usage: %s <rzx file(s)>\n", progname, progname );
+    fprintf( stderr, "Try `%s --help' for more information.\n", progname );
     return 1;
   }
 
-  for( i=1; i<argc; i++ ) {
-    do_file( argv[i] );
+  for( i = 0; i < argc; i++ ) {
+    error = do_file( argv[i] );
+    if( error ) return error;
   }    
 
   return 0;
@@ -130,6 +206,7 @@ do_file( const char *filename )
 
   printf( "Found RZX signature\n" );
   ptr += strlen( rzx_signature );
+  block_number = 0;
 
   printf( "  Major version: %d\n", (int)*ptr++ );
   printf( "  Minor version: %d\n", (int)*ptr++ );
@@ -202,6 +279,8 @@ read_snapshot_block( unsigned char **ptr, unsigned char *end )
 
   printf( "Found a snapshot block\n" );
 
+  printf( "  Block: %lu\n", (unsigned long)block_number ); block_number++;
+
   block_length = read_dword( ptr );
   printf( "  Length: %lu bytes\n", (unsigned long)block_length );
 
@@ -230,6 +309,8 @@ read_input_block( unsigned char **ptr, unsigned char *end )
   }
 
   printf( "Found an input recording block\n" );
+
+  printf( "  Block: %lu\n", (unsigned long)block_number ); block_number++;
   
   length = read_dword( ptr );
   printf( "  Length: %ld bytes\n", (unsigned long)length );
