@@ -1,7 +1,7 @@
 /* rzxtool.c: Simple modifications to RZX files
-   Copyright (c) 2007 Philip Kendall
+   Copyright (c) 2007-2015 Philip Kendall, Sergio Baldovi
 
-   $Id: rzxtool.c 4624 2012-01-09 20:59:35Z pak21 $
+   $Id: rzxtool.c 5424 2016-04-29 13:27:54Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include <config.h>
 
 #include <errno.h>
+#include <getopt.h>
 #include <unistd.h>
 #include <string.h>
 
@@ -36,6 +37,8 @@
 
 #include "utils.h"
 
+#define PROGRAM_NAME "rzxtool"
+
 const char *progname;
 static libspectrum_creator *creator;
 
@@ -44,6 +47,7 @@ typedef enum action_type_t {
   ACTION_DELETE_BLOCK,
   ACTION_EXTRACT_SNAP,
   ACTION_INSERT_SNAP,
+  ACTION_FINALISE_RZX,
 
 } action_type_t;
 
@@ -110,6 +114,7 @@ write_file( unsigned char *buffer, size_t length, const char *filename )
   if( bytes != length ) {
     fprintf( stderr, "%s: wrote only %lu of %lu bytes to `%s'\n", progname,
 	     (unsigned long)bytes, (unsigned long)length, filename );
+    fclose( f );
     return 1;
   }
 
@@ -178,7 +183,28 @@ extract_snap( libspectrum_rzx *rzx, size_t where, const char *filename )
 static libspectrum_snap*
 read_snap( const char *filename )
 {
-  return NULL;
+  unsigned char *buffer = NULL;
+  size_t length = 0;
+  int error;
+  libspectrum_snap *snap;
+
+  if( read_file( filename, &buffer, &length ) ) {
+    return NULL;
+  }
+
+  snap = libspectrum_snap_alloc();
+
+  error = libspectrum_snap_read( snap, buffer, length, LIBSPECTRUM_ID_UNKNOWN,
+                                 filename );
+
+  if( error ) {
+    libspectrum_snap_free( snap );
+    snap = NULL;
+  }
+
+  free( buffer );
+
+  return snap;
 }
 
 static int
@@ -187,7 +213,10 @@ insert_snap( libspectrum_rzx *rzx, size_t where, const char *filename )
   libspectrum_snap *snap;
 
   snap = read_snap( filename );
-  if( !snap ) return 1;
+  if( !snap ) {
+    fprintf( stderr, "%s: couldn't read `%s'\n", progname, filename );
+    return 1;
+  }
 
   libspectrum_rzx_insert_snap( rzx, snap, where );
 
@@ -211,6 +240,9 @@ apply_action( void *data, void *user_data )
   case ACTION_INSERT_SNAP:
     insert_snap( rzx, where, action->filename );
     break;
+  case ACTION_FINALISE_RZX:
+    libspectrum_rzx_finalise( rzx );
+    break;
   default:
     fprintf( stderr, "%s: unknown action type %d\n", progname, action->type );
   }
@@ -235,6 +267,7 @@ parse_argument( const char *argument, int *where, const char **filename )
   if( !comma ) {
     fprintf( stderr, "%s: no comma found in argument `%s'\n", progname,
 	     argument );
+    free( buffer );
     return 1;
   }
 
@@ -268,6 +301,9 @@ add_action( GSList **actions, action_type_t type, const char *argument )
   if( type == ACTION_DELETE_BLOCK ) {
     action->where = atoi( argument );
     action->filename = NULL;
+  } else if( type == ACTION_FINALISE_RZX ) {
+    action->where = 0;
+    action->filename = NULL;
   } else {
 
     error = parse_argument( argument, &where, &filename );
@@ -288,6 +324,46 @@ add_action( GSList **actions, action_type_t type, const char *argument )
   return 0;
 }
 
+static void
+show_version( void )
+{
+  printf(
+    PROGRAM_NAME " (" PACKAGE ") " PACKAGE_VERSION "\n"
+    "Copyright (c) 2007-2014 Philip Kendall\n"
+    "License GPLv2+: GNU GPL version 2 or later "
+    "<http://gnu.org/licenses/gpl.html>\n"
+    "This is free software: you are free to change and redistribute it.\n"
+    "There is NO WARRANTY, to the extent permitted by law.\n" );
+}
+
+static void
+show_help( void )
+{
+  printf(
+    "Usage: %s [OPTION]... <rzxfile> [<outfile>]\n"
+    "Modify Sinclair ZX Spectrum input recording files.\n"
+    "\n"
+    "Options:\n"
+    "  -d <block>     Delete block number.\n"
+    "  -e <block>,<filename>\n"
+    "                 The snapshot in the specified block number will be written\n"
+    "                   to the specified filename.\n"
+    "  -i <block>,<filename>\n"
+    "                 The snapshot specified in filename will be added to the file\n"
+    "                   at the position specified by block number.\n"
+    "  -f             Finalise the RZX file, removing any interspersed snapshot.\n"
+    "  -u             Write uncompressed data to the output RZX file.\n"
+    "  -h, --help     Display this help and exit.\n"
+    "  -V, --version  Output version information and exit.\n"
+    "\n"
+    "Report %s bugs to <%s>\n"
+    "%s home page: <%s>\n"
+    "For complete documentation, see the manual page of %s.\n",
+    progname,
+    PROGRAM_NAME, PACKAGE_BUGREPORT, PACKAGE_NAME, PACKAGE_URL, PROGRAM_NAME
+  );  
+}
+
 static int
 parse_options( int argc, char **argv, GSList **actions,
 	       struct options_t *options )
@@ -297,7 +373,14 @@ parse_options( int argc, char **argv, GSList **actions,
 
   options->uncompressed = 0;
 
-  while( ( c = getopt( argc, argv, "d:e:i:u" ) ) != EOF ) {
+  struct option long_options[] = {
+    { "help", 0, NULL, 'h' },
+    { "version", 0, NULL, 'V' },
+    { 0, 0, 0, 0 }
+  };
+
+  while( ( c = getopt_long( argc, argv, "d:e:i:ufhV", long_options,
+                            NULL ) ) != -1 ) {
     switch( c ) {
     case 'd':
       error = add_action( actions, ACTION_DELETE_BLOCK, optarg );
@@ -310,11 +393,28 @@ parse_options( int argc, char **argv, GSList **actions,
       error = add_action( actions, ACTION_INSERT_SNAP, optarg );
       output_needed = 1;
       break;
+    case 'f':
+      error = add_action( actions, ACTION_FINALISE_RZX, optarg );
+      output_needed = 1;
+      break;
     case 'u':
       options->uncompressed = 1;
       output_needed = 1;
       break;
-    case '?': error = 1; break;
+    case 'h':
+      show_help();
+      exit( 0 );
+    case 'V':
+      show_version();
+      exit( 0 );
+    case '?':
+      /* getopt prints an error message to stderr */
+      error = 1;
+      break;
+    default:
+      error = 1;
+      fprintf( stderr, "%s: unknown option `%c'\n", progname, (char) c );
+      break;
     }
 
     if( error ) break;
@@ -370,19 +470,23 @@ write_rzx( const char *filename, libspectrum_rzx *rzx, int compressed )
 int
 main( int argc, char *argv[] )
 {
-  progname = argv[0];
   GSList *actions = NULL;
   options_t options;
   unsigned char *buffer = NULL; size_t length = 0;
   libspectrum_rzx *rzx;
   int error;
 
+  progname = argv[0];
+
   error = init_libspectrum(); if( error ) return error;
 
   error = get_creator( &creator, "rzxtool" ); if( error ) return error;
 
   error = parse_options( argc, argv, &actions, &options );
-  if( error ) return error;
+  if( error ) {
+    fprintf( stderr, "Try `%s --help' for more information.\n", progname );
+    return error;
+  }
 
   error = read_file( options.rzxfile, &buffer, &length );
   if( error ) return error;
