@@ -1,7 +1,8 @@
 /* scl2trd.c: Convert .SCL disk images to .TRD disk images
    Copyright (c) 2002-2007 Dmitry Sanarin, Philip Kendall and Fredrick Meunier
+   Copyright (c) 2014-2015 Sergio Baldovi
 
-   $Id: scl2trd.c 4889 2013-02-18 23:15:03Z sbaldovi $
+   $Id: scl2trd.c 5424 2016-04-29 13:27:54Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,6 +26,7 @@
 
 #include <config.h>
 
+#include <getopt.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -37,6 +39,8 @@
 #include "compat.h"
 #include "libspectrum.h"
 
+#define PROGRAM_NAME "scl2trd"
+
 struct options {
 
   char *sclfile;		/* The SCL file we'll operate on */
@@ -82,7 +86,7 @@ ui2lsb(unsigned char *mem, unsigned int value)
   mem[3] = ret.b.b3;
 }
 
-static void 
+static int
 Scl2Trd(char *oldname, char *newname)
 {
   int TRD, SCL, i;
@@ -124,9 +128,9 @@ Scl2Trd(char *oldname, char *newname)
   fh = fopen(newname, "wb");
   if (fh) {
     mem = (unsigned char *) malloc(BLOCKSIZE);
-    memset(mem, 0, BLOCKSIZE);
 
     if (mem) {
+      memset(mem, 0, BLOCKSIZE);
       memcpy(&mem[TRD_DIRSTART], template, TRD_DIRLEN);
       strncpy((char*)&mem[TRD_NAMEOFFSET], "Fuse", TRD_MAXNAMELENGTH);
       fwrite((void *) mem, 1, BLOCKSIZE, fh);
@@ -136,13 +140,13 @@ Scl2Trd(char *oldname, char *newname)
 	fwrite((void *) mem, 1, BLOCKSIZE, fh);
 
       free(mem);
-      fclose(fh);
     }
+    fclose(fh);
   }
 
   if ((TRD = open(newname, O_RDWR | O_BINARY)) == -1) {
     printf("Error - cannot open TRD disk image %s !\n", newname);
-    return;
+    return 1;
   }
 
   TRDh = malloc(4096);
@@ -151,7 +155,7 @@ Scl2Trd(char *oldname, char *newname)
     printf("Error - cannot read TRD header from %s\n", newname);
     close(TRD);
     free(TRDh);
-    return;
+    return 1;
   }
 
   tmp = (char *) TRDh + 0x8E5;
@@ -269,18 +273,51 @@ Scl2Trd(char *oldname, char *newname)
 Finish:
   lseek(TRD, 0L, SEEK_SET);
   bytes_written = write(TRD, TRDh, 4096);
-  if (bytes_written < 4096) {
-    printf("Error - writing header to TRD file %s\n", newname);
-  }
   close(TRD);
   free(TRDh);
-  return;
+  if (bytes_written < 4096) {
+    printf("Error - writing header to TRD file %s\n", newname);
+    return 1;
+  }
+  return 0;
 
 Abort:
   close(SCL);
   close(TRD);
   free(TRDh);
   free(tmpscl);
+  return 1;
+}
+
+static void
+show_version( void )
+{
+  printf(
+    PROGRAM_NAME " (" PACKAGE ") " PACKAGE_VERSION "\n"
+    "Copyright (c) 2002-2007 Dmitry Sanarin, Philip Kendall and Fredrick Meunier\n"
+    "License GPLv2+: GNU GPL version 2 or later "
+    "<http://gnu.org/licenses/gpl.html>\n"
+    "This is free software: you are free to change and redistribute it.\n"
+    "There is NO WARRANTY, to the extent permitted by law.\n" );
+}
+
+static void
+show_help( void )
+{
+  printf(
+    "Usage: %s [OPTION] <sclfile> <trdfile>\n"
+    "Convert .scl disk images to .trd disk images.\n"
+    "\n"
+    "Options:\n"
+    "  -h, --help     Display this help and exit.\n"
+    "  -V, --version  Output version information and exit.\n"
+    "\n"
+    "Report %s bugs to <%s>\n"
+    "%s home page: <%s>\n"
+    "For complete documentation, see the manual page of %s.\n",
+    progname,
+    PROGRAM_NAME, PACKAGE_BUGREPORT, PACKAGE_NAME, PACKAGE_URL, PROGRAM_NAME
+  );  
 }
 
 static int
@@ -293,25 +330,48 @@ parse_options(int argc, char **argv, struct options * options)
 
   int unknown = 0;
 
-  while ((c = getopt(argc, argv, "")) != EOF)
-    switch (c) {
+  struct option long_options[] = {
+    { "help", 0, NULL, 'h' },
+    { "version", 0, NULL, 'V' },
+    { 0, 0, 0, 0 }
+  };
+
+  while( ( c = getopt_long( argc, argv, "hV", long_options, NULL ) ) != -1 ) {
+
+    switch( c ) {
+
+    case 'h': show_help(); exit( 0 );
+
+    case 'V': show_version(); exit( 0 );
+
     case '?':
-      unknown = c;
+      /* getopt prints an error message to stderr */
+      unknown = 1;
       break;
+
+    default:
+      unknown = 1;
+      fprintf( stderr, "%s: unknown option `%c'\n", progname, (char) c );
+      break;
+
     }
+  }
+  argc -= optind;
+  argv += optind;
 
   if( unknown ) return 1;
 
-  if (argv[optind] == NULL) {
+  if( argc < 1 ) {
     fprintf(stderr, "%s: no SCL file given\n", progname);
     return 1;
   }
-  options->sclfile = argv[optind];
-  if (argv[optind+1] == NULL) {
+  options->sclfile = argv[0];
+
+  if( argc < 2 ) {
     fprintf(stderr, "%s: no TRD file given\n", progname);
     return 1;
   }
-  options->trdfile = argv[optind+1];
+  options->trdfile = argv[1];
 
   return 0;
 }
@@ -327,14 +387,19 @@ int
 main(int argc, char **argv)
 {
   struct options options;
+  int error;
 
   progname = argv[0];
 
   init_options(&options);
-  if (parse_options(argc, argv, &options))
-    return 1;
+  error = parse_options( argc, argv, &options );
+  if( error ) {
+    fprintf( stderr, "Try `%s --help' for more information.\n", progname );
+    return error;
+  }
 
-  Scl2Trd(options.sclfile, options.trdfile);
+  error = Scl2Trd( options.sclfile, options.trdfile );
+  if( error ) return error;
 
   return 0;
 }
