@@ -3,8 +3,9 @@
                            Martijn van der Heide,
 		 2003 Tomaz Kac,
 		 2003 Philip Kendall
+   Copyright (c) 2014-2015 Sergio Baldovi
 
-   $Id: snap2tzx.c 4754 2012-10-26 13:07:27Z fredm $
+   $Id: snap2tzx.c 5424 2016-04-29 13:27:54Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,6 +29,7 @@
 
 #include <config.h>
 
+#include <getopt.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,6 +41,8 @@
 #include "compat.h"
 #include "utils.h"
 
+#define PROGRAM_NAME "snap2tzx"
+
 char *progname;			/* argv[0] */
 
 typedef struct settings_t {
@@ -163,7 +167,8 @@ static void crunch_z80 (libspectrum_byte *BufferIn, libspectrum_word BlLength, l
   *CrunchedLength = IndexOut;
 }
 
-int test_decz80(libspectrum_byte *source, int final_len, int source_len)
+static int
+test_decz80(libspectrum_byte *source, int final_len, int source_len)
 {
   /* source is not reversed !!! */
 
@@ -307,37 +312,11 @@ setup_string( char *dest, size_t length, const char *src )
 }
 
 static void
-print_usage( int title )
+print_help( void )
 {
-  char *buffer, *buffer2;
-
-  buffer = strdup( progname );
-  if( !buffer ) {
-    print_error( "out of memory at %s:%d", __FILE__, __LINE__ );
-    return;
-  }
-
-  buffer2 = compat_basename( buffer );
-  
-  if( title ) {
-    printf(
-	    "\n"
-	    "snap2tzx: snapshot to TZX converter\n"
-	    "\n"
-	    "Copyright (c) 1997-2001 ThunderWare Research Center,\n"
-	    "              2003 Tomaz Kac,\n"
-	    "              2003 Philip Kendall.\n"
-	    "\n"
-	    "This program is distributed in the hope that it will be useful, but WITHOUT\n"
-	    "ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or\n"
-	    "FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for\n"
-	    "more details.\n"
-	  );
-  }
-
   printf(
-	 "\n"
-	  "Usage: %s [options] <snapshot>\n"
+	  "Usage: %s [OPTION...] <snapshot>\n"
+	  "Converts a ZX Spectrum snapshot into a .tzx tape image.\n"
 	  "\n"
 	  "Options:\n"
 	  "\n"
@@ -351,24 +330,52 @@ print_usage( int title )
 	  "  -s n  Set loading speed: 0: 1500 1: 2250 2:3000 3: 6000 bps\n"
 	  "  -v    Verbose output\n"
 	  "  -$ f  Use .scr file 'f' as the loading screen\n"
+	  "  -h    Display this help and exit\n"
+	  "  -V    Output version information and exit\n"
 	  "\n"
 	  "In string parameters, the copyright symbol will be substituted for '~'.\n"
-	  "\n",
-	  buffer2
+	  "\n"
+	  "Report %s bugs to <%s>\n"
+	  "%s home page: <%s>\n"
+	  "For complete documentation, see the manual page of %s.\n",
+    progname,
+    PROGRAM_NAME, PACKAGE_BUGREPORT, PACKAGE_NAME, PACKAGE_URL, PROGRAM_NAME
 	);
 }
 
+static void
+print_version( void )
+{
+  printf(
+   PROGRAM_NAME " (" PACKAGE ") " PACKAGE_VERSION "\n"
+   "Copyright (c) 1997-2001 ThunderWare Research Center, written by\n"
+   "                        Martijn van der Heide,\n"
+   "Copyright (c) 2003 Tomaz Kac,\n"
+   "Copyright (c) 2003 Philip Kendall\n"
+   "License GPLv2+: GNU GPL version 2 or later "
+   "<http://gnu.org/licenses/gpl.html>\n"
+   "This is free software: you are free to change and redistribute it.\n"
+   "There is NO WARRANTY, to the extent permitted by law.\n" );
+}
+
 static int
 parse_args( settings_t *settings, int argc, char **argv )
 {
   int c, got_game_name, got_loader_name, got_out_filename, error;
   char *buffer, *buffer2, *last_dot;
 
+  struct option long_options[] = {
+    { "help", 0, NULL, 'h' },
+    { "version", 0, NULL, 'V' },
+    { 0, 0, 0, 0 }
+  };
+
   error = initialise_settings( settings ); if( error ) return error;
 
   got_game_name = got_loader_name = got_out_filename = 0;
 
-  while( ( c = getopt( argc, argv, "1:2:b:g:l:o:rs:v$:" ) ) != EOF ) {
+  while( ( c = getopt_long( argc, argv, "1:2:b:g:l:o:rs:v$:hV", long_options,
+                            NULL ) ) != -1 ) {
     switch (c) {
 
     case '1':
@@ -427,16 +434,24 @@ parse_args( settings_t *settings, int argc, char **argv )
       /* External Loading Screen */
     case '$': settings->external_filename = optarg; break;
 
+    case 'h': print_help(); exit( 0 );
+
+    case 'V': print_version(); exit( 0 );
+
     case '?':
-      print_error( "unknown option" );
-      print_usage( 0 );
+      /* getopt prints an error message to stderr */
+      return 1;
+
+    default:
+      print_error( "%s: unknown option `%c'\n", progname, (char) c );
       return 1;
 
     }
   }
 
   if( argv[optind] == NULL ) {
-    print_usage( 1 );
+    fprintf( stderr, "%s: usage: %s [OPTION...] <snapshot>\n", progname,
+             progname );
     return 1;
   }
 
@@ -1165,6 +1180,7 @@ get_loading_screen( libspectrum_byte *screen, const char *filename )
       print_error( "could read only %lu bytes from '%s'", (unsigned long)count,
 		   filename );
     }
+    fclose( f );
     return 1;
   }
 
@@ -1465,6 +1481,7 @@ write_tape( libspectrum_tape *tape, const char *filename )
 		   (unsigned long)written, (unsigned long)length, filename );
     }
     free( buffer );
+    fclose( f );
     return 1;
   }
 
@@ -1510,7 +1527,11 @@ main( int argc, char **argv )
   progname = argv[0];
 
   error = init_libspectrum(); if( error ) return error;
-  error = parse_args( &settings, argc, argv ); if( error ) return error;
+  error = parse_args( &settings, argc, argv );
+  if( error ) {
+    fprintf( stderr, "Try `%s --help' for more information.\n", progname );
+    return error;
+  }
 
   error = load_snap( &snap, settings.input_filename );
   if( error ) return error;
