@@ -1,7 +1,8 @@
 /* tape2wav.c: Convert tape files (tzx, tap, etc.) to .wav files
    Copyright (c) 2007 Fredrick Meunier
+   Copyright (c) 2014-2015 Sergio Baldovi
 
-   $Id: tape2wav.c 4437 2011-05-14 14:19:14Z fredm $
+   $Id: tape2wav.c 5424 2016-04-29 13:27:54Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,6 +28,7 @@
 
 #include <errno.h>
 #include <fcntl.h>
+#include <getopt.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
@@ -37,27 +39,47 @@
 
 #include "utils.h"
 
+#define PROGRAM_NAME "tape2wav"
+
+static void show_help( void );
+static void show_version( void );
 static int read_tape( char *filename, libspectrum_tape **tape );
 static int write_tape( char *filename, libspectrum_tape *tape );
 
 char *progname;
-int sample_rate = 41000;
+int sample_rate = 44100;
 
 int
 main( int argc, char **argv )
 {
-  int c, error;
+  int c, error = 0;
   libspectrum_tape *tzx;
 
   progname = argv[0];
 
-  error = init_libspectrum(); if( error ) return error;
+  struct option long_options[] = {
+    { "help", 0, NULL, 'h' },
+    { "version", 0, NULL, 'V' },
+    { 0, 0, 0, 0 }
+  };
 
-  while( ( c = getopt( argc, argv, "r:" ) ) != -1 ) {
+  while( ( c = getopt_long( argc, argv, "r:hV", long_options, NULL ) ) != -1 ) {
 
     switch( c ) {
 
-    case 'r': sample_rate = abs( atol( optarg ) ); break;
+    case 'r': sample_rate = abs( atoi( optarg ) ); break;
+    case 'h': show_help(); return 0;
+    case 'V': show_version(); return 0;
+
+    case '?':
+      /* getopt prints an error message to stderr */
+      error = 1;
+      break;
+
+    default:
+      error = 1;
+      fprintf( stderr, "%s: unknown option `%c'\n", progname, (char) c );
+      break;
 
     }
 
@@ -66,14 +88,22 @@ main( int argc, char **argv )
   argc -= optind;
   argv += optind;
 
+  if( error ) {
+    fprintf( stderr, "Try `%s --help' for more information.\n", progname );
+    return error;
+  }
+
   if( argc < 2 ) {
     fprintf( stderr,
              "%s: usage: %s [-r rate] <infile> <outfile>\n",
              progname,
 	     progname );
+    fprintf( stderr, "Try `%s --help' for more information.\n", progname );
     return 1;
   }
 
+  error = init_libspectrum(); if( error ) return error;
+
   if( read_tape( argv[0], &tzx ) ) return 1;
 
   if( write_tape( argv[1], tzx ) ) {
@@ -86,6 +116,39 @@ main( int argc, char **argv )
   return 0;
 }
 
+static void
+show_version( void )
+{
+  printf(
+    PROGRAM_NAME " (" PACKAGE ") " PACKAGE_VERSION "\n"
+    "Copyright (c) 2007 Fredrick Meunier\n"
+    "License GPLv2+: GNU GPL version 2 or later "
+    "<http://gnu.org/licenses/gpl.html>\n"
+    "This is free software: you are free to change and redistribute it.\n"
+    "There is NO WARRANTY, to the extent permitted by law.\n" );
+}
+
+static void
+show_help( void )
+{
+  printf(
+    "Usage: %s [OPTION] <infile> <outfile>\n"
+    "Converts ZX Spectrum tape images to audio files.\n"
+    "\n"
+    "Options:\n"
+    "  -r <sample rate>  Set the sample rate for the target wav file. Defaults to\n"
+    "                    44100 Hz.\n"
+    "  -h, --help     Display this help and exit.\n"
+    "  -V, --version  Output version information and exit.\n"
+    "\n"
+    "Report %s bugs to <%s>\n"
+    "%s home page: <%s>\n"
+    "For complete documentation, see the manual page of %s.\n",
+    progname,
+    PROGRAM_NAME, PACKAGE_BUGREPORT, PACKAGE_NAME, PACKAGE_URL, PROGRAM_NAME
+  );
+}
+
 static int
 read_tape( char *filename, libspectrum_tape **tape )
 {
@@ -109,7 +172,7 @@ read_tape( char *filename, libspectrum_tape **tape )
 static int
 write_tape( char *filename, libspectrum_tape *tape )
 {
-  libspectrum_byte *buffer; size_t length;
+  libspectrum_byte *buffer, *ptr; size_t length;
   size_t tape_length = 0;
   libspectrum_error error;
   short level = 0; /* The last level output to this block */
@@ -176,8 +239,10 @@ write_tape( char *filename, libspectrum_tape *tape )
 
     while( tape_length + pulse_length > length ) {
       length *= 2;
+      ptr = buffer;
       buffer = realloc( buffer, length );
       if( !buffer ) {
+        free( ptr );
         fprintf( stderr, "%s: unable to allocate memory for conversion buffer\n",
                  progname );
         return 1;
