@@ -1,7 +1,8 @@
 /* tapeconv.c: Convert .tzx files to .tap files
    Copyright (c) 2002-2008 Philip Kendall, Fredrick Meunier
+   Copyright (c) 2015 Sergio Baldovi
 
-   $Id: tapeconv.c 4734 2012-09-23 05:45:23Z fredm $
+   $Id: tapeconv.c 5424 2016-04-29 13:27:54Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,6 +27,7 @@
 #include <config.h>
 
 #include <errno.h>
+#include <getopt.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
@@ -34,8 +36,11 @@
 
 #include "utils.h"
 
+#define PROGRAM_NAME "tapeconv"
 #define DESCRIPTION_BUFFER_LEN 0x10
 
+static void show_help( void );
+static void show_version( void );
 static int get_type_from_string( libspectrum_id_t *type, const char *string );
 static int read_tape( char *filename, libspectrum_tape **tape );
 static int update_archive_file( char *archive_file, libspectrum_tape *tape );
@@ -57,14 +62,18 @@ char *inlay_file = NULL;
 int
 main( int argc, char **argv )
 {
-  int c, error;
+  int c, error = 0;
   libspectrum_tape *tzx;
 
   progname = argv[0];
 
-  error = init_libspectrum(); if( error ) return error;
+  struct option long_options[] = {
+    { "help", 0, NULL, 'h' },
+    { "version", 0, NULL, 'V' },
+    { 0, 0, 0, 0 }
+  };
 
-  while( ( c = getopt( argc, argv, "a:s:i:b" ) ) != -1 ) {
+  while( ( c = getopt_long( argc, argv, "a:s:i:bhV", long_options, NULL ) ) != -1 ) {
 
     switch( c ) {
 
@@ -72,6 +81,18 @@ main( int argc, char **argv )
     case 'b': beautify = 1; break;
     case 's': scr_file = optarg ; break;
     case 'i': inlay_file = optarg ; break;
+    case 'h': show_help(); return 0;
+    case 'V': show_version(); return 0;
+
+    case '?':
+      /* getopt prints an error message to stderr */
+      error = 1;
+      break;
+
+    default:
+      error = 1;
+      fprintf( stderr, "%s: unknown option `%c'\n", progname, (char) c );
+      break;
 
     }
 
@@ -80,15 +101,23 @@ main( int argc, char **argv )
   argc -= optind;
   argv += optind;
 
+  if( error ) {
+    fprintf( stderr, "Try `%s --help' for more information.\n", progname );
+    return error;
+  }
+
   if( argc < 2 ) {
     fprintf( stderr,
              "%s: usage: %s [-s <scr file>] [-a <archive info tzx>] "
              "[-b] [-i <inlay image>] <infile> <outfile>\n",
              progname,
 	     progname );
+    fprintf( stderr, "Try `%s --help' for more information.\n", progname );
     return 1;
   }
 
+  error = init_libspectrum(); if( error ) return error;
+
   if( read_tape( argv[0], &tzx ) ) return 1;
 
   if( archive_file && update_archive_file( archive_file, tzx ) ) {
@@ -121,6 +150,47 @@ main( int argc, char **argv )
   return 0;
 }
 
+static void
+show_version( void )
+{
+  printf( PROGRAM_NAME " (" PACKAGE ") " PACKAGE_VERSION "\n"
+    "Copyright (c) 2002-2008 Philip Kendall, Fredrick Meunier\n"
+    "License GPLv2+: GNU GPL version 2 or later "
+    "<http://gnu.org/licenses/gpl.html>\n"
+    "This is free software: you are free to change and redistribute it.\n"
+    "There is NO WARRANTY, to the extent permitted by law.\n" );
+}
+
+static void
+show_help( void )
+{
+  printf(
+    "Usage: %s [OPTION]... <infile> <outfile>\n"
+    "Converts between ZX Spectrum tape images.\n"
+    "\n"
+    "Options:\n"
+    "  -a <archive info TZX>\n"
+    "                 Scan the TZX supplied for a Archive Info block and dump\n"
+    "                   to the output TZX.\n"
+    "  -b             Strip output TZX of any concatenation blocks that get created\n"
+    "                   when appending TZX files.\n"
+    "  -s <SCR loading screen>\n"
+    "                 Attach a .scr file to the TZX as a Custom Info block marked\n"
+    "                   as a loading screen\n"
+    "  -i <JPG or GIF inlay image>\n"
+    "                 Attach an image file to the TZX as a Custom Info block marked\n"
+    "                   as a cassette inlay.\n"
+    "  -h, --help     Display this help and exit.\n"
+    "  -V, --version  Output version information and exit.\n"
+    "\n"
+    "Report %s bugs to <%s>\n"
+    "%s home page: <%s>\n"
+    "For complete documentation, see the manual page of %s.\n",
+    progname,
+    PROGRAM_NAME, PACKAGE_BUGREPORT, PACKAGE_NAME, PACKAGE_URL, PROGRAM_NAME
+  );
+}
+
 static int
 get_type_from_string( libspectrum_id_t *type, const char *string )
 {
