@@ -1,7 +1,9 @@
 /* tzxlist.c: Produce a listing of the blocks in a .tzx file
-   Copyright (c) 2001-2011 Philip Kendall, Darren Salt, Fredrick Meunier
+   Copyright (c) 2001-2016 Philip Kendall, Darren Salt, Fredrick Meunier
+   Copyright (c) 2015 Sergio Baldovi
+   Copyright (c) 2016 Gergely Szasz
 
-   $Id: tzxlist.c 4530 2011-07-17 04:38:48Z fredm $
+   $Id: tzxlist.c 5424 2016-04-29 13:27:54Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,6 +29,7 @@
 
 #include <errno.h>
 #include <fcntl.h>
+#include <getopt.h>
 #ifdef HAVE_ICONV
 #include <iconv.h>
 #endif /* #ifdef HAVE_ICONV */
@@ -34,6 +37,7 @@
 #include <locale.h>
 #endif /* #ifdef HAVE_LOCALE_H */
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -42,10 +46,14 @@
 
 #include "utils.h"
 
+#define PROGRAM_NAME "tzxlist"
 #define DESCRIPTION_LENGTH 80
 
 const char *progname;
 
+static int dump_block = -9999;
+static int block_num = 0;
+
 #ifdef HAVE_ICONV
 static iconv_t conversion_descriptor;
 #endif /* #ifdef HAVE_ICONV */
@@ -193,47 +201,118 @@ check_checksum(unsigned long length, libspectrum_byte * data)
 }
 
 static void
-print_block_name( libspectrum_byte * data )
+fprint_block_name( FILE *f, libspectrum_byte *data )
 {
   int i;
   for( i = 2; i < 12; i++) {
-    printf("%c", data[i]);
+    char c;
+    if( data[i] >= 32 && data[i] < 127 ) {
+      c = data[i];
+    } else {
+      c = '?';
+    }
+    fprintf( f, "%c", c );
   }
 }
 
+static void
+print_block_name( libspectrum_byte * data )
+{
+  fprint_block_name( stdout, data );
+}
+
+/* decode header and save data */
 static void
 decode_header( libspectrum_tape_block *block )
 {
   unsigned long length = libspectrum_tape_block_data_length( block );
   libspectrum_byte * data = libspectrum_tape_block_data( block );
   int is_header = length == 19 && data[0] == 0x00;
+  char filename[17]; /* %8d.dsc; %8d.hdr %8d.dat */
+  int save = 0;
+  static int save_data = -9999;
+  FILE *f = NULL;
   if(is_header) {
+    save = ( dump_block == -1 || dump_block == block_num ) ? 1 : 0;
+    if( save ) {
+      snprintf( filename, 16, "%08d.dsc", block_num );
+      f = fopen( filename, "wb" );
+      if( !f ) {
+        fprintf( stderr, "%s: couldn't open '%s': %s\n", progname, filename,
+          strerror( errno ) );
+      }
+    }
     switch( data[1] ) {
     case 0:
       printf("  Program: ");
       print_block_name( data );
+      if( f ) {
+        fprintf( f, "Program: " );
+        fprint_block_name( f, data );
+      }
       break;
     case 1:
       printf("  Number Array: ");
       print_block_name( data );
+      if( f ) {
+        fprintf( f, "Number Array: " );
+        fprint_block_name( f, data );
+      }
       break;
     case 2:
       printf("  Character Array: ");
       print_block_name( data );
+      if( f ) {
+        fprintf( f, "Character Array: " );
+        fprint_block_name( f, data );
+      }
       break;
     case 3:
       printf("  CODE: ");
       print_block_name( data );
       printf("\n  Start: %d\n", data[14] + data[15] * 0x100 );
       printf("  Length: %d", data[12] + data[13] * 0x100 );
+      if( f ) {
+        fprintf( f, "CODE: " );
+        fprint_block_name( f, data );
+        fprintf( f, "\nStart: %d\n", data[14] + data[15] * 0x100 );
+        fprintf( f, "Length: %d", data[12] + data[13] * 0x100 );
+      }
       break;
     default:
       printf("  Unknown");
+      if( f ) {
+        fprintf( f, "Unknown" );
+      }
       break;
     }
     printf("\n");
+    if( f ) {
+      fprintf( f, "\n" );
+      fclose( f );
+    }
   }
   check_checksum(length, data);
+
+  /* save data block */
+  if( save || save_data == block_num - 1 || dump_block == block_num ) {
+
+    snprintf( filename, 16, "%08d.%s",
+              save || dump_block == block_num ? block_num : save_data,
+              save ? "hdr" : "dat" );
+    f = fopen( filename, "wb" );
+    if( !f ) {
+      fprintf( stderr, "%s: couldn't open '%s': %s\n", progname, filename,
+               strerror( errno ) );
+    } else if( fwrite( data + 1, 1, length - 2, f ) != length - 2 ) {
+      fprintf( stderr, "%s: error writing to '%s'\n", progname, filename );
+      fclose( f );
+    } else {
+      fclose( f );
+      printf( "* Tape block dumped\n" );
+    }
+  }
+  save_data = save ? block_num : -9999;
 }
 
 static int
@@ -245,7 +324,7 @@ process_tape( char *filename )
   libspectrum_tape *tape;
   libspectrum_tape_iterator iterator;
   libspectrum_tape_block *block;
-  libspectrum_dword total_length = 0;
+  libspectrum_qword total_length = 0;
   libspectrum_id_t type;
 
   size_t i;
@@ -303,6 +382,7 @@ process_tape( char *filename )
       libspectrum_tape_block_description( description, DESCRIPTION_LENGTH,
 					  block );
     if( error ) return 1;
+    printf( "--= Block #%d =--\n", block_num );
     printf( "Block type 0x%02x (%s)\n", libspectrum_tape_block_type( block ),
 	    description );
     printf("  Block duration: %.2f sec\n",
@@ -510,6 +590,7 @@ process_tape( char *filename )
                8 );
       printf("  Tail length: %d tstates\n",
 	     libspectrum_tape_block_tail_length( block ) );
+      decode_header( block );
       break;
 
     default:
@@ -519,6 +600,8 @@ process_tape( char *filename )
     }
 
     block = libspectrum_tape_iterator_next( &iterator );
+    printf( "\n" );
+    block_num++;
 
   }
 
@@ -536,25 +619,104 @@ process_tape( char *filename )
   return 0;
 }
 
+static void
+show_version( void )
+{
+  printf(
+    PROGRAM_NAME " (" PACKAGE ") " PACKAGE_VERSION "\n"
+    "Copyright (c) 2001-2011 Philip Kendall, Darren Salt, Fredrick Meunier\n"
+    "License GPLv2+: GNU GPL version 2 or later "
+    "<http://gnu.org/licenses/gpl.html>\n"
+    "This is free software: you are free to change and redistribute it.\n"
+    "There is NO WARRANTY, to the extent permitted by law.\n" );
+}
+
+static void
+show_help( void )
+{
+  printf(
+    "Usage: %s [OPTION] <tapefile>...\n"
+    "Outputs a description of the contents of TZX, TAP, PZX and Warajevo tape files.\n"
+    "\n"
+    "Options:\n"
+    "  -d <num>       Dump standard block (header and data) number <num>.\n"
+    "                   Use -1 to dump all blocks.\n"
+    "  -h, --help     Display this help and exit.\n"
+    "  -V, --version  Output version information and exit.\n"
+    "\n"
+    "Report %s bugs to <%s>\n"
+    "%s home page: <%s>\n"
+    "For complete documentation, see the manual page of %s.\n",
+    progname,
+    PROGRAM_NAME, PACKAGE_BUGREPORT, PACKAGE_NAME, PACKAGE_URL, PROGRAM_NAME
+  );
+}
+
 int
 main( int argc, char **argv )
 {
   int ret = 0;
-  int arg = 0;
-  int error;
+  int i;
+  int c, error = 0;
+
+  struct option long_options[] = {
+    { "help", 0, NULL, 'h' },
+    { "version", 0, NULL, 'V' },
+    { 0, 0, 0, 0 }
+  };
 
   progname = argv[0];
 
-  if( argc < 2 ) {
-    fprintf( stderr, "%s: usage: %s <tape files>...\n", progname, progname );
+  while( ( c = getopt_long( argc, argv, "d:hV", long_options, NULL ) ) != -1 ) {
+
+    switch( c ) {
+
+    case 'd':
+      dump_block = atoi( optarg );
+      break;
+
+    case 'h':
+      show_help();
+      return 0;
+
+    case 'V':
+      show_version();
+      return 0;
+
+    case '?':
+      /* getopt prints an error message to stderr */
+      error = 1;
+      break;
+
+    default:
+      error = 1;
+      fprintf( stderr, "%s: unknown option `%c'\n", progname, (char) c );
+      break;
+
+    }
+  }
+
+  argc -= optind;
+  argv += optind;
+
+  if( error ) {
+    fprintf( stderr, "Try `%s --help' for more information.\n", progname );
+    return error;
+  }
+
+  if( argc < 1 ) {
+    fprintf( stderr, "%s: usage: %s [OPTION] <tape files>...\n", progname,
+             progname );
+    fprintf( stderr, "Try `%s --help' for more information.\n", progname );
     return 1;
   }
 
   error = init_libspectrum(); if( error ) return error;
 
 
-  while( ++arg < argc )
-    ret |= process_tape( argv[arg] );
+  for( i = 0; i < argc; i++ ) {
+    ret |= process_tape( argv[i] );
+  }
 
   return ret;
 }
